'use strict';

var language = require('@codemirror/language');
var lezer = require('@surrealdb/lezer');
var common = require('@lezer/common');
var javascript = require('@lezer/javascript');

const surrealqlLanguage = language.LRLanguage.define({
    name: "surrealql",
    parser: lezer.parser.configure({
        props: [
            language.indentNodeProp.add({
                Object: language.continuedIndent({ except: /^\s*}/ }),
                Array: language.continuedIndent({ except: /^\s*]/ }),
            }),
            language.foldNodeProp.add({
                "Object Array CombinedResult": language.foldInside,
            }),
        ],
        wrap: common.parseMixed((node) => {
            return node.name === "JavaScriptContent" ? { parser: javascript.parser } : null;
        }),
    }),
    languageData: {
        closeBrackets: { brackets: ["[", "{", '"', "'", "("] },
        indentOnInput: /^\s*[\]}]$/,
        commentTokens: { line: "--" },
    },
});
const scopeMap = new Map([
    ["permission", "PermissionInput"],
    ["index", "IndexInput"],
    ["combined-results", "CombinedResults"],
    ["syntax", "Syntax"],
]);
/**
 * The CodeMirror extension used to add support for the SurrealQL language
 *
 * @param scope Limit the scope of the highlighting
 */
function surrealql(scope) {
    if (!scope) {
        return new language.LanguageSupport(surrealqlLanguage);
    }
    const scopeId = scopeMap.get(scope);
    if (!scopeId) {
        throw new Error(`Unknown language scope: ${scope}`);
    }
    return new language.LanguageSupport(surrealqlLanguage.configure({ top: scopeId }));
}

exports.surrealql = surrealql;
exports.surrealqlLanguage = surrealqlLanguage;
