import { LRLanguage, indentNodeProp, continuedIndent, foldNodeProp, foldInside, LanguageSupport } from '@codemirror/language';
import { parser } from '@surrealdb/lezer';
import { parseMixed } from '@lezer/common';
import { parser as parser$1 } from '@lezer/javascript';

const surrealqlLanguage = /*@__PURE__*/LRLanguage.define({
    name: "surrealql",
    parser: /*@__PURE__*/parser.configure({
        props: [
            /*@__PURE__*/indentNodeProp.add({
                Object: /*@__PURE__*/continuedIndent({ except: /^\s*}/ }),
                Array: /*@__PURE__*/continuedIndent({ except: /^\s*]/ }),
            }),
            /*@__PURE__*/foldNodeProp.add({
                "Object Array CombinedResult": foldInside,
            }),
        ],
        wrap: /*@__PURE__*/parseMixed((node) => {
            return node.name === "JavaScriptContent" ? { parser: parser$1 } : null;
        }),
    }),
    languageData: {
        closeBrackets: { brackets: ["[", "{", '"', "'", "("] },
        indentOnInput: /^\s*[\]}]$/,
        commentTokens: { line: "--" },
    },
});
const scopeMap = /*@__PURE__*/new Map([
    ["permission", "PermissionInput"],
    ["index", "IndexInput"],
    ["combined-results", "CombinedResults"],
    ["syntax", "Syntax"],
]);
/**
 * The CodeMirror extension used to add support for the SurrealQL language
 *
 * @param scope Limit the scope of the highlighting
 */
function surrealql(scope) {
    if (!scope) {
        return new LanguageSupport(surrealqlLanguage);
    }
    const scopeId = scopeMap.get(scope);
    if (!scopeId) {
        throw new Error(`Unknown language scope: ${scope}`);
    }
    return new LanguageSupport(surrealqlLanguage.configure({ top: scopeId }));
}

export { surrealql, surrealqlLanguage };
